@@ -17,9 +17,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $LastChangedBy: bastian $
- * $LastChangedDate: 2007-05-12 17:42:00 +0000 (Sa, 12 Mai 2007) $
- * $LastChangedRevision: 1413 $
+ * $LastChangedBy$
+ * $LastChangedDate$
+ * $LastChangedRevision$
  */
 
 #ifndef CHECK_H
@@ -29,6 +29,5 @@
 
 int check_deb (const char *target, di_package *package, const char *message);
 int check_packages (const char *target, const char *ext, di_release *rel);
-int check_release (const char *file, const char *sig);
 
 #endif
