@@ -17,9 +17,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $LastChangedBy: bastian $
- * $LastChangedDate: 2008-06-24 12:22:06 +0200 (Di, 24 Jun 2008) $
- * $LastChangedRevision: 1668 $
+ * $LastChangedBy$
+ * $LastChangedDate$
+ * $LastChangedRevision$
  */
 
 #define _GNU_SOURCE
@@ -226,7 +226,7 @@ static void check_target (const char *target)
   else if (errno == ENOENT)
   {
     if (mkdir (target, 0777))
-      log_text (DI_LOG_LEVEL_ERROR, "Failed to creae target");
+      log_text (DI_LOG_LEVEL_ERROR, "Failed to create target");
   }
   else
     log_text (DI_LOG_LEVEL_ERROR, "Target check failed: %s", strerror (errno));
@@ -394,7 +394,7 @@ static int foreign_finish ()
 
   fputs ("\
 rm /sbin/init\n\
-dpkg-divert --remove --local --divert /sbin/init.REAL /sbin/init\n\
+dpkg-divert --remove --local --rename --divert /sbin/init.REAL /sbin/init\n\
 rm /sbin/cdebootstrap-foreign\n\
 ", foreign_script);
 
@@ -545,7 +545,7 @@ int frontend_main (int argc, char **argv, char **envp __attribute((unused)))
 
   configdir = helperdir = keyringdirs[1] = generate_configdir ();
 
-  while ((c = getopt_long (argc, argv, "a:c:df:hH:i:s:qv", long_opts, NULL)) != -1)
+  while ((c = getopt_long (argc, argv, "a:c:df:hH:i:k:s:qv", long_opts, NULL)) != -1)
   {
     switch (c)
     {
